package solutions.bitbadger.documents.groovy.jvm.integration.common

import solutions.bitbadger.documents.Field
import solutions.bitbadger.documents.groovy.support.JsonDocument
import solutions.bitbadger.documents.support.ThrowawayDatabase

import static solutions.bitbadger.documents.extensions.ConnExt.*
import static solutions.bitbadger.documents.support.TypesKt.TEST_TABLE
import static org.junit.jupiter.api.Assertions.*

class CountFunctions {

    static void all(ThrowawayDatabase db) {
        JsonDocument.load(db)
        assertEquals 5L, countAll(db.conn, TEST_TABLE), 'There should have been 5 documents in the table'
    }

    static void byFieldsNumeric(ThrowawayDatabase db) {
        JsonDocument.load(db)
        assertEquals(3L, countByFields(db.conn, TEST_TABLE, List.of(Field.between('numValue', 10, 20))),
                'There should have been 3 matching documents')
    }

    static void byFieldsAlpha(ThrowawayDatabase db) {
        JsonDocument.load(db)
        assertEquals(1L, countByFields(db.conn, TEST_TABLE, List.of(Field.between('value', 'aardvark', 'apple'))),
                'There should have been 1 matching document')
    }

    static void byContainsMatch(ThrowawayDatabase db) {
        JsonDocument.load(db)
        assertEquals(2L, countByContains(db.conn, TEST_TABLE, Map.of('value', 'purple')),
                'There should have been 2 matching documents')
    }

    static void byContainsNoMatch(ThrowawayDatabase db) {
        JsonDocument.load(db)
        assertEquals(0L, countByContains(db.conn, TEST_TABLE, Map.of('value', 'magenta')),
                'There should have been no matching documents')
    }

    static void byJsonPathMatch(ThrowawayDatabase db) {
        JsonDocument.load(db)
        assertEquals(2L, countByJsonPath(db.conn, TEST_TABLE, '$.numValue ? (@ < 5)'),
                'There should have been 2 matching documents')
    }

    static void byJsonPathNoMatch(ThrowawayDatabase db) {
        JsonDocument.load(db)
        assertEquals(0L, countByJsonPath(db.conn, TEST_TABLE, '$.numValue ? (@ > 100)'),
                'There should have been no matching documents')
    }
}
