package solutions.bitbadger.documents.groovy.jvm.integration.common

import solutions.bitbadger.documents.Configuration
import solutions.bitbadger.documents.Field
import solutions.bitbadger.documents.Parameter
import solutions.bitbadger.documents.ParameterType
import solutions.bitbadger.documents.groovy.support.JsonDocument
import solutions.bitbadger.documents.jvm.Results
import solutions.bitbadger.documents.query.CountQuery
import solutions.bitbadger.documents.query.DeleteQuery
import solutions.bitbadger.documents.query.FindQuery
import solutions.bitbadger.documents.support.ThrowawayDatabase

import static org.junit.jupiter.api.Assertions.*
import static solutions.bitbadger.documents.extensions.ConnExt.*
import static solutions.bitbadger.documents.support.TypesKt.TEST_TABLE

class CustomFunctions {

    static void listEmpty(ThrowawayDatabase db) {
        JsonDocument.load(db)
        deleteByFields(db.conn, TEST_TABLE, List.of(Field.exists(Configuration.idField)))
        def result = customList(db.conn, FindQuery.all(TEST_TABLE), List.of(), JsonDocument.class, Results.&fromData)
        assertEquals(0, result.size(), "There should have been no results")
    }

    static void listAll(ThrowawayDatabase db) {
        JsonDocument.load(db)
        def result = customList(db.conn, FindQuery.all(TEST_TABLE), List.of(), JsonDocument.class, Results.&fromData)
        assertEquals(5, result.size(), "There should have been 5 results")
    }

    static void singleNone(ThrowawayDatabase db) {
        assertNull(customSingle(db.conn, FindQuery.all(TEST_TABLE), List.of(), JsonDocument.class, Results.&fromData),
                "There should not have been a document returned")

    }

    static void singleOne(ThrowawayDatabase db) {
        JsonDocument.load(db)
        assertNotNull(
                customSingle(db.conn, FindQuery.all(TEST_TABLE), List.of(), JsonDocument.class, Results.&fromData),
                "There should not have been a document returned")
    }

    static void nonQueryChanges(ThrowawayDatabase db) {
        JsonDocument.load(db)
        assertEquals(5L, customScalar(db.conn, CountQuery.all(TEST_TABLE), List.of(), Long.class, Results.&toCount),
                "There should have been 5 documents in the table")
        customNonQuery(db.conn, "DELETE FROM $TEST_TABLE")
        assertEquals(0L, customScalar(db.conn, CountQuery.all(TEST_TABLE), List.of(), Long.class, Results.&toCount),
                "There should have been no documents in the table")
    }

    static void nonQueryNoChanges(ThrowawayDatabase db) {
        JsonDocument.load(db)
        assertEquals(5L, customScalar(db.conn, CountQuery.all(TEST_TABLE), List.of(), Long.class, Results.&toCount),
                "There should have been 5 documents in the table")
        customNonQuery(db.conn, DeleteQuery.byId(TEST_TABLE, "eighty-two"),
                List.of(new Parameter(":id", ParameterType.STRING, "eighty-two")))
        assertEquals(5L, customScalar(db.conn, CountQuery.all(TEST_TABLE), List.of(), Long.class, Results.&toCount),
                "There should still have been 5 documents in the table")
    }

    static void scalar(ThrowawayDatabase db) {
        JsonDocument.load(db)
        assertEquals(3L,
                customScalar(db.conn, "SELECT 3 AS it FROM $TEST_TABLE LIMIT 1", List.of(), Long.class,
                        Results.&toCount),
                "The number 3 should have been returned")
    }
}
