package solutions.bitbadger.documents.groovy.jvm.integration.sqlite

import org.junit.jupiter.api.DisplayName
import org.junit.jupiter.api.Test
import solutions.bitbadger.documents.DocumentException
import solutions.bitbadger.documents.groovy.jvm.integration.common.CountFunctions
import solutions.bitbadger.documents.jvm.integration.sqlite.SQLiteDB

import static org.junit.jupiter.api.Assertions.assertThrows

/**
 * SQLite integration tests for the `Count` object / `count*` connection extension functions
 */
@DisplayName("JVM | Groovy | SQLite: Count")
class CountIT {

    @Test
    @DisplayName("all counts all documents")
    void all() {
        new SQLiteDB().withCloseable { CountFunctions.all(it) }
    }

    @Test
    @DisplayName("byFields counts documents by a numeric value")
    void byFieldsNumeric() {
        new SQLiteDB().withCloseable { CountFunctions.byFieldsNumeric(it) }
    }

    @Test
    @DisplayName("byFields counts documents by a alphanumeric value")
    void byFieldsAlpha() {
        new SQLiteDB().withCloseable { CountFunctions.byFieldsAlpha(it) }
    }

    @Test
    @DisplayName("byContains fails")
    void byContainsMatch() {
        new SQLiteDB().withCloseable { db ->
            assertThrows(DocumentException) { CountFunctions.byContainsMatch(db) }
        }
    }

    @Test
    @DisplayName("byJsonPath fails")
    void byJsonPathMatch() {
        new SQLiteDB().withCloseable { db ->
            assertThrows(DocumentException) { CountFunctions.byJsonPathMatch(db) }
        }
    }
}
