package solutions.bitbadger.documents.java.jvm.integration.common;

import solutions.bitbadger.documents.*;
import solutions.bitbadger.documents.java.support.JsonDocument;
import solutions.bitbadger.documents.jvm.Results;
import solutions.bitbadger.documents.query.CountQuery;
import solutions.bitbadger.documents.query.DeleteQuery;
import solutions.bitbadger.documents.query.FindQuery;
import solutions.bitbadger.documents.support.ThrowawayDatabase;

import java.util.Collection;
import java.util.List;

import static org.junit.jupiter.api.Assertions.*;
import static solutions.bitbadger.documents.extensions.ConnExt.*;
import static solutions.bitbadger.documents.support.TypesKt.TEST_TABLE;

final public class CustomFunctions {

    public static void listEmpty(ThrowawayDatabase db) throws DocumentException {
        JsonDocument.load(db);
        deleteByFields(db.getConn(), TEST_TABLE, List.of(Field.exists(Configuration.idField)));
        Collection<JsonDocument> result =
                customList(db.getConn(), FindQuery.all(TEST_TABLE), List.of(), JsonDocument.class, Results::fromData);
        assertEquals(0, result.size(), "There should have been no results");
    }

    public static void listAll(ThrowawayDatabase db) throws DocumentException {
        JsonDocument.load(db);
        Collection<JsonDocument> result =
                customList(db.getConn(), FindQuery.all(TEST_TABLE), List.of(), JsonDocument.class, Results::fromData);
        assertEquals(5, result.size(), "There should have been 5 results");
    }

    public static void singleNone(ThrowawayDatabase db) throws DocumentException {
        assertNull(
                customSingle(db.getConn(), FindQuery.all(TEST_TABLE), List.of(), JsonDocument.class, Results::fromData),
                "There should not have been a document returned");
    }

    public static void singleOne(ThrowawayDatabase db) throws DocumentException {
        JsonDocument.load(db);
        assertNotNull(
                customSingle(db.getConn(), FindQuery.all(TEST_TABLE), List.of(), JsonDocument.class, Results::fromData),
                "There should not have been a document returned");
    }

    public static void nonQueryChanges(ThrowawayDatabase db) throws DocumentException {
        JsonDocument.load(db);
        assertEquals(5L,
                customScalar(db.getConn(), CountQuery.all(TEST_TABLE), List.of(), Long.class, Results::toCount),
                "There should have been 5 documents in the table");
        customNonQuery(db.getConn(), String.format("DELETE FROM %s", TEST_TABLE));
        assertEquals(0L,
                customScalar(db.getConn(), CountQuery.all(TEST_TABLE), List.of(), Long.class, Results::toCount),
                "There should have been no documents in the table");
    }

    public static void nonQueryNoChanges(ThrowawayDatabase db) throws DocumentException {
        JsonDocument.load(db);
        assertEquals(5L,
                customScalar(db.getConn(), CountQuery.all(TEST_TABLE), List.of(), Long.class, Results::toCount),
                "There should have been 5 documents in the table");
        customNonQuery(db.getConn(), DeleteQuery.byId(TEST_TABLE, "eighty-two"),
                List.of(new Parameter<>(":id", ParameterType.STRING, "eighty-two")));
        assertEquals(5L,
                customScalar(db.getConn(), CountQuery.all(TEST_TABLE), List.of(), Long.class, Results::toCount),
                "There should still have been 5 documents in the table");
    }

    public static void scalar(ThrowawayDatabase db) throws DocumentException {
        JsonDocument.load(db);
        assertEquals(3L,
                customScalar(db.getConn(), String.format("SELECT 3 AS it FROM %s LIMIT 1", TEST_TABLE), List.of(),
                        Long.class, Results::toCount),
                "The number 3 should have been returned");
    }
}
