package solutions.bitbadger.documents.java.jvm.integration.postgresql;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import solutions.bitbadger.documents.DocumentException;
import solutions.bitbadger.documents.jvm.integration.postgresql.PgDB;
import solutions.bitbadger.documents.java.jvm.integration.common.CustomFunctions;

/**
 * PostgreSQL integration tests for the `Custom` object / `custom*` connection extension functions
 */
@DisplayName("JVM | Java | PostgreSQL: Custom")
final public class CustomIT {

    @Test
    @DisplayName("list succeeds with empty list")
    public void listEmpty() throws DocumentException {
        try (PgDB db = new PgDB()) {
            CustomFunctions.listEmpty(db);
        }
    }

    @Test
    @DisplayName("list succeeds with a non-empty list")
    public void listAll() throws DocumentException {
        try (PgDB db = new PgDB()) {
            CustomFunctions.listAll(db);
        }
    }

    @Test
    @DisplayName("single succeeds when document not found")
    public void singleNone() throws DocumentException {
        try (PgDB db = new PgDB()) {
            CustomFunctions.singleNone(db);
        }
    }

    @Test
    @DisplayName("single succeeds when a document is found")
    public void singleOne() throws DocumentException {
        try (PgDB db = new PgDB()) {
            CustomFunctions.singleOne(db);
        }
    }

    @Test
    @DisplayName("nonQuery makes changes")
    public void nonQueryChanges() throws DocumentException {
        try (PgDB db = new PgDB()) {
            CustomFunctions.nonQueryChanges(db);
        }
    }

    @Test
    @DisplayName("nonQuery makes no changes when where clause matches nothing")
    public void nonQueryNoChanges() throws DocumentException {
        try (PgDB db = new PgDB()) {
            CustomFunctions.nonQueryNoChanges(db);
        }
    }

    @Test
    @DisplayName("scalar succeeds")
    public void scalar() throws DocumentException {
        try (PgDB db = new PgDB()) {
            CustomFunctions.scalar(db);
        }
    }
}
