package solutions.bitbadger.documents.scala.jvm.integration.common

import org.junit.jupiter.api.Assertions.*
import solutions.bitbadger.documents.Field
import solutions.bitbadger.documents.extensions.ConnExt.*
import solutions.bitbadger.documents.scala.support.JsonDocument
import solutions.bitbadger.documents.support.ThrowawayDatabase
import solutions.bitbadger.documents.support.TypesKt.TEST_TABLE

import scala.jdk.CollectionConverters.*

object CountFunctions {

  def all(db: ThrowawayDatabase): Unit =
    JsonDocument.load(db)
    assertEquals(5L, countAll(db.getConn, TEST_TABLE), "There should have been 5 documents in the table")

  def byFieldsNumeric(db: ThrowawayDatabase): Unit =
    JsonDocument.load(db)
    assertEquals(3L, countByFields(db.getConn, TEST_TABLE, (Field.between("numValue", 10, 20) :: Nil).asJava),
      "There should have been 3 matching documents")

  def byFieldsAlpha(db: ThrowawayDatabase): Unit =
    JsonDocument.load(db)
    assertEquals(1L, countByFields(db.getConn, TEST_TABLE, (Field.between("value", "aardvark", "apple") :: Nil).asJava),
      "There should have been 1 matching document")

  def byContainsMatch(db: ThrowawayDatabase): Unit =
    JsonDocument.load(db)
    assertEquals(2L, countByContains(db.getConn, TEST_TABLE, Map.Map1("value", "purple")),
      "There should have been 2 matching documents")

  def byContainsNoMatch(db: ThrowawayDatabase): Unit =
    JsonDocument.load(db)
    assertEquals(0L, countByContains(db.getConn, TEST_TABLE, Map.Map1("value", "magenta")),
      "There should have been no matching documents")

  def byJsonPathMatch(db: ThrowawayDatabase): Unit =
    JsonDocument.load(db)
    assertEquals(2L, countByJsonPath(db.getConn, TEST_TABLE, "$.numValue ? (@ < 5)"),
      "There should have been 2 matching documents")

  def byJsonPathNoMatch(db: ThrowawayDatabase): Unit =
    JsonDocument.load(db)
    assertEquals(0L, countByJsonPath(db.getConn, TEST_TABLE, "$.numValue ? (@ > 100)"),
      "There should have been no matching documents")
}
