package solutions.bitbadger.documents.scala.jvm.integration.common

import org.junit.jupiter.api.Assertions.*
import solutions.bitbadger.documents.{Configuration, Field, Parameter, ParameterType}
import solutions.bitbadger.documents.extensions.ConnExt.*
import solutions.bitbadger.documents.jvm.Results
import solutions.bitbadger.documents.query.{CountQuery, DeleteQuery, FindQuery}
import solutions.bitbadger.documents.scala.support.JsonDocument
import solutions.bitbadger.documents.support.ThrowawayDatabase
import solutions.bitbadger.documents.support.TypesKt.TEST_TABLE

import scala.annotation.nowarn
import scala.jdk.CollectionConverters.*

object CustomFunctions {

  def listEmpty(db: ThrowawayDatabase): Unit =
    JsonDocument.load(db)
    deleteByFields(db.getConn, TEST_TABLE, (Field.exists(Configuration.idField) :: Nil).asJava)
    @nowarn
    val result = customList(db.getConn, FindQuery.all(TEST_TABLE), List().asJava, classOf[JsonDocument],
      Results.fromData)
    assertEquals(0, result.size, "There should have been no results")

  def listAll(db: ThrowawayDatabase): Unit =
    JsonDocument.load(db)
    @nowarn
    val result = customList(db.getConn, FindQuery.all(TEST_TABLE), List().asJava, classOf[JsonDocument],
      Results.fromData)
    assertEquals(5, result.size, "There should have been 5 results")

  @nowarn
  def singleNone(db: ThrowawayDatabase): Unit =
    assertNull(
      customSingle(db.getConn, FindQuery.all(TEST_TABLE), List().asJava, classOf[JsonDocument], Results.fromData),
      "There should not have been a document returned")

  @nowarn
  def singleOne(db: ThrowawayDatabase): Unit =
    JsonDocument.load(db)
    assertNotNull(
      customSingle(db.getConn, FindQuery.all(TEST_TABLE), List().asJava, classOf[JsonDocument], Results.fromData),
      "There should not have been a document returned")

  @nowarn
  def nonQueryChanges(db: ThrowawayDatabase): Unit =
    JsonDocument.load(db)
    assertEquals(5L,
      customScalar(db.getConn, CountQuery.all(TEST_TABLE), List().asJava, classOf[Long], Results.toCount),
      "There should have been 5 documents in the table")
    customNonQuery(db.getConn, s"DELETE FROM $TEST_TABLE")
    assertEquals(0L,
      customScalar(db.getConn, CountQuery.all(TEST_TABLE), List().asJava, classOf[Long], Results.toCount),
      "There should have been no documents in the table")

  @nowarn
  def nonQueryNoChanges(db: ThrowawayDatabase): Unit =
    JsonDocument.load(db)
    assertEquals(5L,
      customScalar(db.getConn, CountQuery.all(TEST_TABLE), List().asJava, classOf[Long], Results.toCount),
      "There should have been 5 documents in the table")
    customNonQuery(db.getConn, DeleteQuery.byId(TEST_TABLE, "eighty-two"),
      (Parameter(":id", ParameterType.STRING, "eighty-two") :: Nil).asJava)
    assertEquals(5L,
      customScalar(db.getConn, CountQuery.all(TEST_TABLE), List().asJava, classOf[Long], Results.toCount),
      "There should still have been 5 documents in the table")

  @nowarn
  def scalar(db: ThrowawayDatabase): Unit =
    JsonDocument.load(db)
    assertEquals(3L,
      customScalar(db.getConn, s"SELECT 3 AS it FROM $TEST_TABLE LIMIT 1", List().asJava, classOf[Long],
        Results.toCount),
      "The number 3 should have been returned")
}
